@@ -7,19 +7,21 @@ This program can be distributed under the terms of the GNU GPLv3.
 '''
 
 from __future__ import division, print_function, absolute_import
-
 from ..common import QuietError, BUFSIZE
 from abc import ABCMeta, abstractmethod
 from base64 import b64decode, b64encode
 from cStringIO import StringIO
 from contextlib import contextmanager
+from functools import wraps
 from getpass import getpass
 from pycryptopp.cipher import aes
+from s3ql.common import ChecksumError
 import ConfigParser
 import bz2
 import cPickle as pickle
 import hashlib
 import hmac
+import httplib
 import logging
 import lzma
 import os
@@ -30,7 +32,6 @@ import sys
 import threading
 import time
 import zlib
-from functools import wraps
 
 # Not available in every pycryptopp version
 if hasattr(aes, 'start_up_self_test'):
@@ -40,7 +41,7 @@ log = logging.getLogger("backend")
 
 HMAC_SIZE = 32
 
-RETRY_TIMEOUT=60*60*24
+RETRY_TIMEOUT = 60 * 60 * 24
 def retry(fn):
     '''Decorator for retrying a method on some exceptions
     
@@ -49,10 +50,10 @@ def retry(fn):
     at increasing intervals. If this persists for more than *timeout* seconds,
     the most-recently caught exception is re-raised.
     '''
-    
+
     @wraps(fn)
-    def wrapped(self, *a, **kw):    
-        interval = 1/50
+    def wrapped(self, *a, **kw):
+        interval = 1 / 50
         waited = 0
         while True:
             try:
@@ -63,18 +64,20 @@ def retry(fn):
                 if not self.is_temp_failure(exc):
                     raise
                 if waited > RETRY_TIMEOUT:
-                    log.error('%s.%s(*): Timeout exceeded, re-raising %r exception', 
+                    log.error('%s.%s(*): Timeout exceeded, re-raising %r exception',
                             self.__class__.__name__, fn.__name__, exc)
                     raise
-                
-                log.info('Encountered %r exception, retrying call to %s.%s...', 
-                          exc, self.__class__.__name__, fn.__name__)
+
+                log.info('Encountered %s exception (%s), retrying call to %s.%s...',
+                          type(exc).__name__, exc, self.__class__.__name__, fn.__name__)
+
+            if hasattr(exc, 'retry_after') and exc.retry_after:
+                interval = exc.retry_after
                 
             time.sleep(interval)
             waited += interval
-            if interval < 20*60:
-                interval *= 2   
-                
+            interval = min(5*60, 2*interval)
+
     # False positive
     #pylint: disable=E1101
     wrapped.__doc__ += '''
@@ -82,9 +85,48 @@ This method has been decorated and will automatically recall itself in
 increasing intervals for up to s3ql.backends.common.RETRY_TIMEOUT seconds if it
 raises an exception for which the instance's `is_temp_failure` method returns
 True.
-'''  
-    return wrapped 
+'''
+    return wrapped
 
+def http_connection(hostname, port, ssl=False):
+    '''Return http connection to *hostname*:*port*
+    
+    This method honors the http_proxy and https_proxy environment
+    variables.
+    '''
+    
+    log.debug('Connecting to %s...', hostname)
+    
+    if 'https_proxy' in os.environ:
+        proxy = os.environ['https_proxy']
+        hit = re.match(r'^(https?://)?([a-zA-Z0-9.-]+)(:[0-9]+)?/?$', proxy)
+        if not hit:
+            log.warn('Unable to parse proxy setting %s', proxy)
+        
+        if hit.group(1) == 'https://':
+            log.warn('HTTPS connection to proxy is probably pointless and not supported, '
+                     'will use standard HTTP')
+        
+        if hit.group(3):
+            proxy_port = int(hit.group(3)[1:])
+        else:
+            proxy_port = 80
+            
+        proxy_host = hit.group(2)
+        log.info('Using proxy %s:%d', proxy_host, proxy_port)
+        
+        if ssl:
+            conn = httplib.HTTPSConnection(proxy_host, proxy_port)
+        else:
+            conn = httplib.HTTPConnection(proxy_host, proxy_port)
+        conn.set_tunnel(hostname, port)
+        return conn
+    
+    elif ssl:
+        return httplib.HTTPSConnection(hostname, port)
+    else:
+        return httplib.HTTPConnection(hostname, port)
+    
 def sha256(s):
     return hashlib.sha256(s).digest()
 
@@ -95,43 +137,43 @@ class BucketPool(object):
     starting with underscore) may be called concurrently by different
     threads.    
     '''
-    
+
     def __init__(self, factory):
         '''Init pool
         
         *factory* should be a callable that provides new
         connections.
         '''
-    
+
         self.factory = factory
         self.pool = []
         self.lock = threading.Lock()
-        
+
     def pop_conn(self):
         '''Pop connection from pool'''
-        
+
         with self.lock:
             if self.pool:
                 return self.pool.pop()
             else:
                 return self.factory()
-                    
+
     def push_conn(self, conn):
         '''Push connection back into pool'''
-        
+
         with self.lock:
             self.pool.append(conn)
-        
+
     @contextmanager
     def __call__(self):
         '''Provide connection from pool (context manager)'''
-        
+
         conn = self.pop_conn()
         try:
             yield conn
         finally:
             self.push_conn(conn)
-            
+
 
 class AbstractBucket(object):
     '''Functionality shared between all backends.
@@ -146,13 +188,13 @@ class AbstractBucket(object):
     __metaclass__ = ABCMeta
 
     needs_login = True
-    
+
     def __init__(self):
         super(AbstractBucket, self).__init__()
 
     def __getitem__(self, key):
         return self.fetch(key)[0]
-        
+
     def __setitem__(self, key, value):
         self.store(key, value)
 
@@ -168,7 +210,7 @@ class AbstractBucket(object):
     def iteritems(self):
         for key in self.list():
             yield (key, self[key])
-    
+
     @retry
     def perform_read(self, fn, key):
         '''Read bucket data using *fn*, retry on temporary failure
@@ -179,7 +221,7 @@ class AbstractBucket(object):
         '''
         with self.open_read(key) as fh:
             return fn(fh)
-             
+
     @retry
     def perform_write(self, fn, key, metadata=None, is_compressed=False):
         '''Read bucket data using *fn*, retry on temporary failure
@@ -188,10 +230,10 @@ class AbstractBucket(object):
         error (as defined by `is_temp_failure`) occurs during opening, closing
         or execution of *fn*, the operation is retried.
         '''
-        
+
         with self.open_write(key, metadata, is_compressed) as fh:
             return fn(fh)
-                
+
     def fetch(self, key):
         """Return data stored under `key`.
 
@@ -200,10 +242,10 @@ class AbstractBucket(object):
         ``bucket.fetch(key)[0]``.
         """
 
-        def do_read(fh): 
+        def do_read(fh):
             data = fh.read()
             return (data, fh.metadata)
-            
+
         return self.perform_read(do_read, key)
 
     def store(self, key, val, metadata=None):
@@ -216,7 +258,7 @@ class AbstractBucket(object):
         bucket instead of using this function: ``bucket[key] = val`` is
         equivalent to ``bucket.store(key, val)``.
         """
-        
+
         self.perform_write(lambda fh: fh.write(val), key, metadata)
 
     @abstractmethod
@@ -232,10 +274,10 @@ class AbstractBucket(object):
         request cannot automatically be retried. In these case this method can
         be used to check for temporary problems and so that the request can
         be manually restarted if applicable.
-        '''   
-               
+        '''
+
         pass
-    
+
     @abstractmethod
     def lookup(self, key):
         """Return metadata for given key.
@@ -244,12 +286,12 @@ class AbstractBucket(object):
         """
 
         pass
-    
+
     @abstractmethod
     def get_size(self, key):
         '''Return size of object stored under *key*'''
-        pass 
-    
+        pass
+
     @abstractmethod
     def open_read(self, key):
         """Open object for reading
@@ -259,9 +301,9 @@ class AbstractBucket(object):
         can be modified by the caller at will. The object must be closed
         explicitly. 
         """
-        
+
         pass
-    
+
     @abstractmethod
     def open_write(self, key, metadata=None, is_compressed=False):
         """Open object for writing
@@ -276,7 +318,7 @@ class AbstractBucket(object):
         to write compressed data, and may be used to avoid recompression
         by the bucket.
         """
-        
+
         pass
 
     @abstractmethod
@@ -288,7 +330,7 @@ class AbstractBucket(object):
         retrieval attempts.
         '''
         pass
-                    
+
     @abstractmethod
     def is_list_create_consistent(self):
         '''If True, new objects are guaranteed to show up in object listings
@@ -297,7 +339,7 @@ class AbstractBucket(object):
         reflected when retrieving the list of available objects.
         '''
         pass
-    
+
     @abstractmethod
     def clear(self):
         """Delete all objects in bucket"""
@@ -305,7 +347,7 @@ class AbstractBucket(object):
 
     def contains(self, key):
         '''Check if `key` is in bucket'''
-        
+
         try:
             self.lookup(key)
         except NoSuchObject:
@@ -337,7 +379,7 @@ class AbstractBucket(object):
         If `dest` already exists, it will be overwritten. The copying
         is done on the remote side. 
         """
-        
+
         pass
 
     def rename(self, src, dest):
@@ -346,10 +388,10 @@ class AbstractBucket(object):
         If `dest` already exists, it will be overwritten. The rename
         is done on the remote side.
         """
-        
+
         self.copy(src, dest)
         self.delete(src)
-    
+
 class BetterBucket(AbstractBucket):
     '''
     This class adds encryption, compression and integrity protection to a plain
@@ -358,14 +400,14 @@ class BetterBucket(AbstractBucket):
 
     def __init__(self, passphrase, compression, bucket):
         super(BetterBucket, self).__init__()
-        
+
         self.passphrase = passphrase
         self.compression = compression
         self.bucket = bucket
 
         if compression not in ('bzip2', 'lzma', 'zlib', None):
             raise ValueError('Unsupported compression: %s' % compression)
-        
+
     def lookup(self, key):
         """Return metadata for given key.
 
@@ -375,16 +417,16 @@ class BetterBucket(AbstractBucket):
         metadata = self.bucket.lookup(key)
         convert_legacy_metadata(metadata)
         return self._unwrap_meta(metadata)
-  
+
     def get_size(self, key):
         '''Return size of object stored under *key*
         
         This method returns the compressed size, i.e. the storage space
         that's actually occupied by the object.
         '''
-        
-        return self.bucket.get_size(key) 
-      
+
+        return self.bucket.get_size(key)
+
     def is_temp_failure(self, exc):
         '''Return true if exc indicates a temporary error
     
@@ -397,10 +439,10 @@ class BetterBucket(AbstractBucket):
         request cannot automatically be retried. In these case this method can
         be used to check for temporary problems and so that the request can
         be manually restarted if applicable.
-        '''          
-        
+        '''
+
         return self.bucket.is_temp_failure(exc)
-    
+
     def _unwrap_meta(self, metadata):
         '''Unwrap metadata
         
@@ -420,7 +462,7 @@ class BetterBucket(AbstractBucket):
         buf = b64decode(metadata['meta'])
         if encrypted:
             buf = decrypt(buf, self.passphrase)
-            
+
         try:
             metadata = pickle.loads(buf)
         except pickle.UnpicklingError as exc:
@@ -428,12 +470,12 @@ class BetterBucket(AbstractBucket):
                 and exc.args[0].startswith('invalid load key')):
                 raise ChecksumError('Invalid metadata')
             raise
-            
+
         if metadata is None:
             return dict()
         else:
             return metadata
-        
+
     def open_read(self, key):
         """Open object for reading
 
@@ -447,10 +489,10 @@ class BetterBucket(AbstractBucket):
 
         fh = self.bucket.open_read(key)
         convert_legacy_metadata(fh.metadata)
-        
+
         compr_alg = fh.metadata['compression']
         encr_alg = fh.metadata['encryption']
-           
+
         metadata = self._unwrap_meta(fh.metadata)
 
         if compr_alg == 'BZIP2':
@@ -463,20 +505,20 @@ class BetterBucket(AbstractBucket):
             decompressor = None
         else:
             raise RuntimeError('Unsupported compression: %s' % compr_alg)
-                    
+
         if encr_alg == 'AES':
             fh = LegacyDecryptDecompressFilter(fh, self.passphrase, decompressor)
         else:
             if encr_alg == 'AES_v2':
-                fh = DecryptFilter(fh, self.passphrase)                
+                fh = DecryptFilter(fh, self.passphrase)
             elif encr_alg != 'None':
                 raise RuntimeError('Unsupported encryption: %s' % encr_alg)
-        
+
             if decompressor:
                 fh = DecompressFilter(fh, decompressor)
-        
+
         fh.metadata = metadata
-        
+
         return fh
 
     def open_write(self, key, metadata=None, is_compressed=False):
@@ -492,7 +534,7 @@ class BetterBucket(AbstractBucket):
         to write compressed data, and may be used to avoid recompression
         by the bucket.        
         """
-   
+
         # We always store metadata (even if it's just None), so that we can
         # verify that the object has been created by us when we call lookup().
         meta_buf = pickle.dumps(metadata, 2)
@@ -501,7 +543,7 @@ class BetterBucket(AbstractBucket):
 
         if self.passphrase:
             meta_raw['encryption'] = 'AES_v2'
-            nonce = struct.pack(b'<f', time.time() - time.timezone) + bytes(key)
+            nonce = struct.pack(b'<f', time.time()) + bytes(key)
             meta_raw['meta'] = b64encode(encrypt(meta_buf, self.passphrase, nonce))
         else:
             meta_raw['encryption'] = 'None'
@@ -510,7 +552,7 @@ class BetterBucket(AbstractBucket):
 
         if is_compressed or not self.compression:
             compr = None
-            meta_raw['compression'] = 'None'            
+            meta_raw['compression'] = 'None'
         elif self.compression == 'zlib':
             compr = zlib.compressobj(9)
             meta_raw['compression'] = 'ZLIB'
@@ -527,9 +569,9 @@ class BetterBucket(AbstractBucket):
             fh = EncryptFilter(fh, self.passphrase, nonce)
         if compr:
             fh = CompressFilter(fh, compr)
-            
+
         return fh
-            
+
     def is_get_consistent(self):
         '''If True, objects retrievals are guaranteed to be up-to-date
         
@@ -538,7 +580,7 @@ class BetterBucket(AbstractBucket):
         retrieval attempts.
         '''
         return self.bucket.is_get_consistent()
-                    
+
     def is_list_create_consistent(self):
         '''If True, new objects are guaranteed to show up in object listings
         
@@ -546,7 +588,7 @@ class BetterBucket(AbstractBucket):
         reflected when retrieving the list of available objects.
         '''
         return self.bucket.is_get_consistent()
-    
+
     def clear(self):
         """Delete all objects in bucket"""
         return self.bucket.clear()
@@ -589,21 +631,21 @@ class BetterBucket(AbstractBucket):
 
 class AbstractInputFilter(object):
     '''Process data while reading'''
-    
+
     __metaclass__ = ABCMeta
-    
+
     def __init__(self):
         super(AbstractInputFilter, self).__init__()
         self.buffer = ''
-        
+
     def read(self, size=None):
         '''Try to read *size* bytes
         
         If *None*, read until EOF.
         '''
-        
+
         if size is None:
-            remaining = 1<<31
+            remaining = 1 << 31
         else:
             remaining = size - len(self.buffer)
 
@@ -613,24 +655,24 @@ class AbstractInputFilter(object):
                 break
             remaining -= len(buf)
             self.buffer += buf
-                
+
         if size is None:
             buf = self.buffer
             self.buffer = ''
         else:
             buf = self.buffer[:size]
             self.buffer = self.buffer[size:]
-            
+
         return buf
-             
+
     @abstractmethod
     def _read(self, size):
-        '''Read roughly *size* bytes'''    
+        '''Read roughly *size* bytes'''
         pass
-        
+
 class CompressFilter(object):
     '''Compress data while writing'''
-    
+
     def __init__(self, fh, compr):
         '''Initialize
         
@@ -638,20 +680,20 @@ class CompressFilter(object):
         instance with a *compress* method.
         '''
         super(CompressFilter, self).__init__()
-        
+
         self.fh = fh
         self.compr = compr
         self.obj_size = 0
         self.closed = False
-        
+
     def write(self, data):
         '''Write *data*'''
-        
+
         buf = self.compr.compress(data)
         if buf:
             self.fh.write(buf)
             self.obj_size += len(buf)
-            
+
     def close(self):
         buf = self.compr.flush()
         self.fh.write(buf)
@@ -661,20 +703,20 @@ class CompressFilter(object):
 
     def __enter__(self):
         return self
-    
+
     def __exit__(self, *a):
         self.close()
         return False
-        
+
     def get_obj_size(self):
         if not self.closed:
             raise RuntimeError('Object must be closed first.')
         return self.obj_size
-                
-                
+
+
 class DecompressFilter(AbstractInputFilter):
     '''Decompress data while reading'''
-    
+
     def __init__(self, fh, decomp, metadata=None):
         '''Initialize
         
@@ -682,14 +724,14 @@ class DecompressFilter(AbstractInputFilter):
         fresh decompressor instance with a *decompress* method.
         '''
         super(DecompressFilter, self).__init__()
-        
+
         self.fh = fh
         self.decomp = decomp
         self.metadata = metadata
-        
+
     def _read(self, size):
         '''Read roughly *size* bytes'''
-        
+
         buf = ''
         while not buf:
             buf = self.fh.read(size)
@@ -697,7 +739,7 @@ class DecompressFilter(AbstractInputFilter):
                 if self.decomp.unused_data:
                     raise ChecksumError('Data after end of compressed stream')
                 return ''
-                
+
             try:
                 buf = self.decomp.decompress(buf)
             except IOError as exc:
@@ -714,45 +756,45 @@ class DecompressFilter(AbstractInputFilter):
                              exc.args[0])
                     raise ChecksumError('Invalid compressed stream')
                 raise
-                        
+
         return buf
-    
+
     def close(self):
         self.fh.close()
-        
+
     def __enter__(self):
         return self
-    
+
     def __exit__(self, *a):
         self.close()
         return False
-        
+
 class EncryptFilter(object):
     '''Encrypt data while writing'''
-    
+
     def __init__(self, fh, passphrase, nonce):
         '''Initialize
         
         *fh* should be a file-like object.
         '''
         super(EncryptFilter, self).__init__()
-        
+
         self.fh = fh
         self.obj_size = 0
         self.closed = False
-        
+
         if isinstance(nonce, unicode):
             nonce = nonce.encode('utf-8')
-    
+
         self.key = sha256(passphrase + nonce)
         self.cipher = aes.AES(self.key) #IGNORE:E1102
         self.hmac = hmac.new(self.key, digestmod=hashlib.sha256)
-    
+
         self.fh.write(struct.pack(b'<B', len(nonce)))
         self.fh.write(nonce)
-        
+
         self.obj_size += len(nonce) + 1
-    
+
     def write(self, data):
         '''Write *data*
         
@@ -763,17 +805,17 @@ class EncryptFilter(object):
         reasonable size (if the data is written in e.g. 4 byte chunks, it is
         blown up by 100%)
         '''
-        
+
         if len(data) == 0:
             return
-        
+
         buf = struct.pack(b'<I', len(data)) + data
         self.hmac.update(buf)
         buf = self.cipher.process(buf)
         if buf:
             self.fh.write(buf)
             self.obj_size += len(buf)
-            
+
     def close(self):
         # Packet length of 0 indicates end of stream, only HMAC follows
         buf = struct.pack(b'<I', 0)
@@ -786,71 +828,71 @@ class EncryptFilter(object):
 
     def __enter__(self):
         return self
-    
+
     def __exit__(self, *a):
         self.close()
         return False
-        
+
     def get_obj_size(self):
         if not self.closed:
             raise RuntimeError('Object must be closed first.')
         return self.obj_size
-        
-        
+
+
 class DecryptFilter(AbstractInputFilter):
     '''Decrypt data while reading
     
     Reader has to read the entire stream in order for HMAC
     checking to work.
     '''
-    
+
     def __init__(self, fh, passphrase, metadata=None):
         '''Initialize
         
         *fh* should be a file-like object.
         '''
         super(DecryptFilter, self).__init__()
-        
+
         self.fh = fh
         self.off_size = struct.calcsize(b'<I')
         self.remaining = 0 # Remaining length of current packet
         self.metadata = metadata
         self.hmac_checked = False
-        
+
         # Read nonce
         len_ = struct.unpack(b'<B', fh.read(struct.calcsize(b'<B')))[0]
         nonce = fh.read(len_)
-    
+
         key = sha256(passphrase + nonce)
         self.cipher = aes.AES(key) #IGNORE:E1102
         self.hmac = hmac.new(key, digestmod=hashlib.sha256)
-    
+
     def _read(self, size):
         '''Read roughly *size* bytes'''
-         
+
         buf = self.fh.read(size)
         if not buf:
             if not self.hmac_checked:
                 raise ChecksumError('HMAC mismatch')
             return ''
-        
+
         inbuf = self.cipher.process(buf)
         outbuf = ''
-        while True:  
-        
+        while True:
+
             if len(inbuf) <= self.remaining:
                 self.remaining -= len(inbuf)
                 self.hmac.update(inbuf)
                 outbuf += inbuf
                 break
-            
+
             outbuf += inbuf[:self.remaining]
-            self.hmac.update(inbuf[:self.remaining+self.off_size])
+            self.hmac.update(inbuf[:self.remaining + self.off_size])
             paket_size = struct.unpack(b'<I', inbuf[self.remaining
-                                                    :self.remaining+self.off_size])[0]
+                                                    :self.remaining + self.off_size])[0]
             inbuf = inbuf[self.remaining + self.off_size:]
             self.remaining = paket_size
-            
+
             # End of file, read and check HMAC
             if paket_size == 0:
                 if len(inbuf) != HMAC_SIZE:
@@ -859,50 +901,50 @@ class DecryptFilter(AbstractInputFilter):
                     raise ChecksumError('HMAC mismatch')
                 self.hmac_checked = True
                 break
-        
-        return outbuf                                           
+
+        return outbuf
 
     def close(self):
         self.fh.close()
 
     def __enter__(self):
         return self
-    
+
     def __exit__(self, *a):
         self.close()
-        return False        
-    
+        return False
+
 class LegacyDecryptDecompressFilter(AbstractInputFilter):
     '''Decrypt and Decompress data while reading
     
     Reader has to read the entire stream in order for HMAC
     checking to work.
     '''
-    
+
     def __init__(self, fh, passphrase, decomp, metadata=None):
         '''Initialize
         
         *fh* should be a file-like object.
         '''
         super(LegacyDecryptDecompressFilter, self).__init__()
-        
+
         self.fh = fh
         self.metadata = metadata
         self.decomp = decomp
         self.hmac_checked = False
-        
+
         # Read nonce
         len_ = struct.unpack(b'<B', fh.read(struct.calcsize(b'<B')))[0]
         nonce = fh.read(len_)
         self.hash = fh.read(HMAC_SIZE)
-    
+
         key = sha256(passphrase + nonce)
         self.cipher = aes.AES(key) #IGNORE:E1102
         self.hmac = hmac.new(key, digestmod=hashlib.sha256)
-        
+
     def _read(self, size):
         '''Read roughly *size* bytes'''
-         
+
         buf = None
         while not buf:
             buf = self.fh.read(size)
@@ -916,11 +958,11 @@ class LegacyDecryptDecompressFilter(AbstractInputFilter):
                     return ''
             elif not buf:
                 return ''
-            
+
             buf = self.cipher.process(buf)
             if not self.decomp:
                 break
-            
+
             try:
                 buf = self.decomp.decompress(buf)
             except IOError as exc:
@@ -930,32 +972,27 @@ class LegacyDecryptDecompressFilter(AbstractInputFilter):
             except lzma.error as exc:
                 if exc.args == ('unknown file format',):
                     raise ChecksumError('Invalid compressed stream')
-                raise            
+                raise
             except zlib.error as exc:
                 if exc.args[0].startswith('Error -3 while decompressing:'):
                     log.warn('LegacyDecryptDecompressFilter._read(): %s',
                              exc.args[0])
                     raise ChecksumError('Invalid compressed stream')
                 raise
-            
+
         self.hmac.update(buf)
-        return buf                              
+        return buf
 
     def close(self):
         self.fh.close()
 
     def __enter__(self):
         return self
-    
+
     def __exit__(self, *a):
         self.close()
-        return False       
-    
-    # pickle requires a readline() method for unpickling, but isn't actually
-    # using it if we're using the binary protocols.
-    def readline(self):
-        raise RuntimeError('not implemented')
-    
+        return False
+
 def encrypt(buf, passphrase, nonce):
     '''Encrypt *buf*'''
 
@@ -973,7 +1010,7 @@ def encrypt(buf, passphrase, nonce):
     return b''.join(
                     (struct.pack(b'<B', len(nonce)),
                     nonce, hash_, buf))
-            
+
 def decrypt(buf, passphrase):
     '''Decrypt *buf'''
 
@@ -1000,20 +1037,6 @@ def decrypt(buf, passphrase):
 
     return buf
 
-
-class ChecksumError(Exception):
-    """
-    Raised if there is a checksum error in the data that we received.
-    """
-    
-    def __init__(self, str_):
-        super(ChecksumError, self).__init__()
-        self.str = str_
-    
-    def __str__(self):
-        return self.str
-    
-
 class ObjectNotEncrypted(Exception):
     '''
     Raised by the backend if an object was requested from an encrypted
@@ -1022,31 +1045,51 @@ class ObjectNotEncrypted(Exception):
     We do not want to simply return the uncrypted object, because the
     caller may rely on the objects integrity being cryptographically
     verified.
-    ''' 
+    '''
 
     pass
 
 class NoSuchObject(Exception):
     '''Raised if the requested object does not exist in the bucket'''
-    
+
     def __init__(self, key):
         super(NoSuchObject, self).__init__()
         self.key = key
-        
+
     def __str__(self):
         return 'Bucket does not have anything stored under key %r' % self.key
 
 class NoSuchBucket(Exception):
     '''Raised if the requested bucket does not exist'''
-    
+
     def __init__(self, name):
         super(NoSuchBucket, self).__init__()
         self.name = name
-        
+
     def __str__(self):
         return 'Bucket %r does not exist' % self.name
-        
 
+
+class AuthorizationError(Exception):
+    '''Raised if the credentials don't give access to the requested bucket'''
+
+    def __init__(self, msg):
+        super(AuthorizationError, self).__init__()
+        self.msg = msg
+
+    def __str__(self):
+        return 'Access denied. Server said: %s' % self.msg
+
+class AuthenticationError(Exception):
+    '''Raised if the credentials are invalid'''
+
+    def __init__(self, msg):
+        super(AuthenticationError, self).__init__()
+        self.msg = msg
+
+    def __str__(self):
+        return 'Access denied. Server said: %s' % self.msg
+    
 def convert_legacy_metadata(meta):
     if ('encryption' in meta and
         'compression' in meta):
@@ -1099,28 +1142,27 @@ def get_bucket_factory(options, plain=False):
     If *plain* is true, don't attempt to unlock and don't wrap into
     BetterBucket.    
     '''
-                                     
-    hit = re.match(r'^([a-zA-Z0-9]+)://(.+)$', options.storage_url)
+
+    hit = re.match(r'^([a-zA-Z0-9]+)://', options.storage_url)
     if not hit:
         raise QuietError('Unknown storage url: %s' % options.storage_url)
-    
+
     backend_name = 's3ql.backends.%s' % hit.group(1)
-    bucket_name = hit.group(2)
     try:
         __import__(backend_name)
     except ImportError:
         raise QuietError('No such backend: %s' % hit.group(1))
-    
+
     bucket_class = getattr(sys.modules[backend_name], 'Bucket')
-    
+
     # Read authfile
     config = ConfigParser.SafeConfigParser()
     if os.path.isfile(options.authfile):
         mode = os.stat(options.authfile).st_mode
         if mode & (stat.S_IRGRP | stat.S_IROTH):
-            raise QuietError("%s has insecure permissions, aborting." % options.authfile)    
+            raise QuietError("%s has insecure permissions, aborting." % options.authfile)
         config.read(options.authfile)
-    
+
     backend_login = None
     backend_pw = None
     bucket_passphrase = None
@@ -1132,14 +1174,14 @@ def get_bucket_factory(options, plain=False):
                 return None
 
         pattern = getopt('storage-url')
-        
+
         if not pattern or not options.storage_url.startswith(pattern):
             continue
-        
+
         backend_login = backend_login or getopt('backend-login')
         backend_pw = backend_pw or getopt('backend-password')
         bucket_passphrase = bucket_passphrase or getopt('bucket-passphrase')
-      
+
     if not backend_login and bucket_class.needs_login:
         if sys.stdin.isatty():
             backend_login = getpass("Enter backend login: ")
@@ -1148,44 +1190,61 @@ def get_bucket_factory(options, plain=False):
 
     if not backend_pw and bucket_class.needs_login:
         if sys.stdin.isatty():
-            backend_pw = getpass("Enter backend password: ") 
+            backend_pw = getpass("Enter backend password: ")
         else:
             backend_pw = sys.stdin.readline().rstrip()
 
-    if plain:
-        return lambda: bucket_class(bucket_name, backend_login, backend_pw)
-    
-    bucket = bucket_class(bucket_name, backend_login, backend_pw)
-     
+
+
     try:
-        encrypted = 's3ql_passphrase' in bucket
-    except NoSuchBucket:
-        raise QuietError('Bucket %d does not exist' % bucket_name)
+        bucket = bucket_class(options.storage_url, backend_login, backend_pw)
         
+        # Do not use bucket.lookup(), this would use a HEAD request and
+        # not provide any useful error messages if something goes wrong
+        # (e.g. wrong credentials)
+        _ = bucket['s3ql_passphrase']
+        
+    except NoSuchBucket:
+        raise QuietError('Bucket does not exist')   
+    
+    except AuthorizationError:
+        raise QuietError('No permission to access bucket.')
+
+    except AuthenticationError:
+        raise QuietError('Invalid credentials or skewed system clock.')
+        
+    except NoSuchObject:
+        encrypted = False
+        
+    else:
+        encrypted = True
+        
+    if plain:
+        return lambda: bucket_class(options.storage_url, backend_login, backend_pw)
+            
     if encrypted and not bucket_passphrase:
         if sys.stdin.isatty():
-            bucket_passphrase = getpass("Enter bucket encryption passphrase: ") 
+            bucket_passphrase = getpass("Enter bucket encryption passphrase: ")
         else:
             bucket_passphrase = sys.stdin.readline().rstrip()
     elif not encrypted:
         bucket_passphrase = None
-        
+
     if hasattr(options, 'compress'):
         compress = options.compress
     else:
         compress = None
-            
+
     if not encrypted:
-        return lambda: BetterBucket(None, compress, 
-                                    bucket_class(bucket_name, backend_login, backend_pw))
-    
+        return lambda: BetterBucket(None, compress,
+                                    bucket_class(options.storage_url, backend_login, backend_pw))
+
     tmp_bucket = BetterBucket(bucket_passphrase, compress, bucket)
-    
+
     try:
         data_pw = tmp_bucket['s3ql_passphrase']
     except ChecksumError:
         raise QuietError('Wrong bucket passphrase')
 
-    return lambda: BetterBucket(data_pw, compress, 
-                                bucket_class(bucket_name, backend_login, backend_pw))
-    
\ No newline at end of file
+    return lambda: BetterBucket(data_pw, compress,
+                                bucket_class(options.storage_url, backend_login, backend_pw))
