@@ -25,15 +25,16 @@ class Bucket(AbstractBucket):
     '''
 
     needs_login = False
-    
-    def __init__(self, name, backend_login, backend_pw): #IGNORE:W0613
+
+    def __init__(self, storage_url, backend_login, backend_pw): #IGNORE:W0613
         '''Initialize local bucket
         
         Login and password are ignored.
         '''
         super(Bucket, self).__init__()
+        name = storage_url[len('local://'):]
         self.name = name
-        
+
         if not os.path.exists(name):
             raise NoSuchBucket(name)
 
@@ -42,15 +43,15 @@ class Bucket(AbstractBucket):
 
     def is_temp_failure(self, exc): #IGNORE:W0613
         '''Return true if exc indicates a temporary error'''
-        
+
         return False
-    
+
     def lookup(self, key):
         """Return metadata for given key.
 
         If the key does not exist, `NoSuchObject` is raised.
         """
-        
+
         path = self._key_to_path(key)
         try:
             with open(path, 'rb') as src:
@@ -65,12 +66,12 @@ class Bucket(AbstractBucket):
                 and exc.args[0].startswith('invalid load key')):
                 raise ChecksumError('Invalid metadata')
             raise
-        
+
     def get_size(self, key):
         '''Return size of object stored under *key*'''
-        
-        return os.path.getsize(self._key_to_path(key)) 
-            
+
+        return os.path.getsize(self._key_to_path(key))
+
     def open_read(self, key):
         """Open object for reading
 
@@ -78,16 +79,16 @@ class Bucket(AbstractBucket):
         the file-like object, metadata is stored in its *metadata* attribute and
         can be modified by the caller at will.
         """
-        
+
         path = self._key_to_path(key)
         try:
-            fh = ObjectR(path) 
+            fh = ObjectR(path)
         except IOError as exc:
             if exc.errno == errno.ENOENT:
                 raise NoSuchObject(key)
             else:
                 raise
-            
+
         try:
             fh.metadata = pickle.load(fh)
         except pickle.UnpicklingError as exc:
@@ -96,7 +97,7 @@ class Bucket(AbstractBucket):
                 raise ChecksumError('Invalid metadata')
             raise
         return fh
-    
+
     def open_write(self, key, metadata=None, is_compressed=False):
         """Open object for writing
 
@@ -110,17 +111,17 @@ class Bucket(AbstractBucket):
         to write compressed data, and may be used to avoid recompression
         by the bucket.           
         """
-        
+
         if metadata is None:
             metadata = dict()
-        
-        
+
+
         path = self._key_to_path(key)
 
         # By renaming, we make sure that there are no
         # conflicts between parallel reads, the last one wins
-        tmpname = '%s#%d-%d' % (path, os.getpid(), thread.get_ident()) 
-        
+        tmpname = '%s#%d-%d' % (path, os.getpid(), thread.get_ident())
+
         try:
             dest = ObjectW(tmpname)
         except IOError as exc:
@@ -135,11 +136,11 @@ class Bucket(AbstractBucket):
                     # Another thread may have created the directory already
                     pass
             dest = ObjectW(tmpname)
-            
+
         os.rename(tmpname, path)
         pickle.dump(metadata, dest, 2)
-        return dest    
-            
+        return dest
+
     def is_get_consistent(self):
         '''If True, objects retrievals are guaranteed to be up-to-date
         
@@ -147,9 +148,9 @@ class Bucket(AbstractBucket):
         object is guaranteed to be immediately reflected in subsequent object
         retrieval attempts.
         '''
-        
+
         return True
-                    
+
     def is_list_create_consistent(self):
         '''If True, new objects are guaranteed to show up in object listings
         
@@ -158,10 +159,10 @@ class Bucket(AbstractBucket):
         '''
 
         return True
-    
+
     def clear(self):
         """Delete all objects in bucket"""
-        
+
         for name in os.listdir(self.name):
             path = os.path.join(self.name, name)
             if os.path.isdir(path):
@@ -171,7 +172,7 @@ class Bucket(AbstractBucket):
 
     def contains(self, key):
         '''Check if `key` is in bucket'''
-        
+
         path = self._key_to_path(key)
         try:
             os.lstat(path)
@@ -205,27 +206,27 @@ class Bucket(AbstractBucket):
         Returns an iterator over all keys in the bucket.
         '''
         if prefix:
-            base = os.path.dirname(self._key_to_path(prefix))     
+            base = os.path.dirname(self._key_to_path(prefix))
         else:
             base = self.name
-            
+
         for (path, dirnames, filenames) in os.walk(base, topdown=True):
-            
+
             # Do not look in wrong directories
             if prefix:
                 rpath = path[len(self.name):] # path relative to base
                 prefix_l = ''.join(rpath.split('/'))
-                
+
                 dirs_to_walk = list()
                 for name in dirnames:
                     prefix_ll = unescape(prefix_l + name)
                     if prefix_ll.startswith(prefix[:len(prefix_ll)]):
                         dirs_to_walk.append(name)
                 dirnames[:] = dirs_to_walk
-                                            
+
             for name in filenames:
                 key = unescape(name)
-                
+
                 if not prefix or key.startswith(prefix):
                     yield key
 
@@ -234,7 +235,7 @@ class Bucket(AbstractBucket):
         
         If `dest` already exists, it will be overwritten.
         """
-        
+
         path_src = self._key_to_path(src)
         path_dest = self._key_to_path(dest)
 
@@ -252,9 +253,9 @@ class Bucket(AbstractBucket):
                     raise
                 else:
                     # Another thread may have created the directory already
-                    pass            
+                    pass
             dest = open(path_dest, 'wb')
-        
+
         try:
             with open(path_src, 'rb') as src:
                 shutil.copyfileobj(src, dest, BUFSIZE)
@@ -275,8 +276,8 @@ class Bucket(AbstractBucket):
         dest_path = self._key_to_path(dest)
         if not os.path.exists(src_path):
             raise NoSuchObject(src)
-           
-        try: 
+
+        try:
             os.rename(src_path, dest_path)
         except OSError as exc:
             if exc.errno != errno.ENOENT:
@@ -288,26 +289,26 @@ class Bucket(AbstractBucket):
                     raise
                 else:
                     # Another thread may have created the directory already
-                    pass                   
-            os.rename(src_path, dest_path)  
-        
+                    pass
+            os.rename(src_path, dest_path)
+
     def _key_to_path(self, key):
         '''Return path for given key'''
-        
+
         # NOTE: We must not split the path in the middle of an
         # escape sequence, or list() will fail to work.
-        
+
         key = escape(key)
-        
+
         if not key.startswith('s3ql_data_'):
             return os.path.join(self.name, key)
-        
+
         no = key[10:]
         path = [ self.name, 's3ql_data_']
         for i in range(0, len(no), 3):
             path.append(no[:i])
         path.append(key)
-        
+
         return os.path.join(*path)
 
 def escape(s):
@@ -330,41 +331,40 @@ def unescape(s):
 
 class ObjectR(file):
     '''A local storage object opened for reading'''
-    
+
     def __init__(self, name, metadata=None):
         super(ObjectR, self).__init__(name, 'rb', buffering=0)
-        self.metadata = metadata     
-        
+        self.metadata = metadata
+
 class ObjectW(object):
     '''A local storage object opened for writing'''
-    
+
     def __init__(self, name):
         super(ObjectW, self).__init__()
         self.fh = open(name, 'wb', 0)
         self.obj_size = 0
         self.closed = False
-        
+
     def write(self, buf):
         '''Write object data'''
-        
+
         self.fh.write(buf)
         self.obj_size += len(buf)
-   
+
     def close(self):
         '''Close object and upload data'''
-        
+
         self.fh.close()
         self.closed = True
-          
+
     def __enter__(self):
         return self
-    
+
     def __exit__(self, *a):
         self.close()
         return False
-    
+
     def get_obj_size(self):
         if not self.closed:
             raise RuntimeError('Object must be closed first.')
-        return self.obj_size   
-    
\ No newline at end of file
+        return self.obj_size
