@@ -7,9 +7,13 @@ This program can be distributed under the terms of the GNU GPLv3.
 '''
 
 from __future__ import division, print_function, absolute_import
+from . import s3
+from ..common import QuietError
+from .common import http_connection
+import re
 
-from . import s3 
-import httplib
+# Pylint goes berserk with false positives
+#pylint: disable=E1002,E1101,W0232
 
 class Bucket(s3.Bucket):
     """A bucket stored in Amazon S3
@@ -20,11 +24,22 @@ class Bucket(s3.Bucket):
     object will be immediately retrievable. Additional consistency guarantees
     may or may not be available and can be queried for with instance methods.    
     """
-            
+
     def _get_conn(self):
         '''Return connection to server'''
         
-        return httplib.HTTPSConnection('%s.s3.amazonaws.com' % self.bucket_name)
-    
+        return http_connection(self.hostname, self.port, ssl=True)
+
     def __str__(self):
-        return 's3s://%s/%s' % (self.bucket_name, self.prefix)            
\ No newline at end of file
+        return 's3s://%s/%s' % (self.bucket_name, self.prefix)
+
+    @staticmethod
+    def _parse_storage_url(storage_url):
+        hit = re.match(r'^s3s://([^/]+)(?:/(.*))?$', storage_url)
+        if not hit:
+            raise QuietError('Invalid storage URL')
+
+        bucket_name = hit.group(1)
+        hostname = '%s.s3.amazonaws.com' % bucket_name
+        prefix = hit.group(2) or ''
+        return (hostname, 443, bucket_name, prefix)
