@@ -9,7 +9,7 @@ This program can be distributed under the terms of the GNU GPLv3.
 from __future__ import division, print_function, absolute_import
 from . import CURRENT_FS_REV
 from .backends.common import get_bucket, BetterBucket
-from .common import (get_bucket_cachedir, setup_logging, QuietError, 
+from .common import (get_bucket_cachedir, setup_logging, QuietError,
     stream_write_bz2, CTRL_INODE)
 from .database import Connection
 from .metadata import dump_metadata, create_tables
@@ -39,7 +39,7 @@ def parse_args(args):
     parser.add_quiet()
     parser.add_version()
     parser.add_storage_url()
-    
+
     parser.add_argument("-L", default='', help="Filesystem label",
                       dest="label", metavar='<name>',)
     parser.add_argument("--max-obj-size", type=int, default=10240, metavar='<size>',
@@ -52,12 +52,12 @@ def parse_args(args):
                         help="Overwrite any existing data.")
 
     options = parser.parse_args(args)
-        
+
     return options
 
 def init_tables(conn):
     # Insert root directory
-    timestamp = time.time() - time.timezone
+    timestamp = time.time()
     conn.execute("INSERT INTO inodes (id,mode,uid,gid,mtime,atime,ctime,refcount) "
                  "VALUES (?,?,?,?,?,?,?,?)",
                    (ROOT_INODE, stat.S_IFDIR | stat.S_IRUSR | stat.S_IWUSR | stat.S_IXUSR
@@ -79,7 +79,7 @@ def init_tables(conn):
                          (b'lost+found', 1))
     conn.execute("INSERT INTO contents (name_id, inode, parent_inode) VALUES(?,?,?)",
                  (name_id, inode, ROOT_INODE))
-    
+
 def main(args=None):
 
     if args is None:
@@ -87,26 +87,26 @@ def main(args=None):
 
     options = parse_args(args)
     setup_logging(options)
-    
+
     if options.max_obj_size < 1024:
         log.warn('Warning: maximum object sizes less than 1 MB will seriously degrade '
                  'performance.')
-        
+
     try:
         plain_bucket = get_bucket(options, plain=True)
     except NoSuchBucket as exc:
         raise QuietError(str(exc))
-    
+
     if 's3ql_metadata' in plain_bucket:
         if not options.force:
             raise QuietError("Found existing file system! Use --force to overwrite")
-            
+
         log.info('Purging existing file system data..')
         plain_bucket.clear()
         if not plain_bucket.is_get_consistent():
             log.info('Please note that the new file system may appear inconsistent\n'
                      'for a while until the removals have propagated through the backend.')
-            
+
     if not options.plain:
         if sys.stdin.isatty():
             wrap_pw = getpass("Enter encryption password: ")
@@ -120,12 +120,12 @@ def main(args=None):
         fh = open('/dev/urandom', "rb", 0) # No buffering
         data_pw = fh.read(32)
         fh.close()
-        
+
         bucket = BetterBucket(wrap_pw, 'bzip2', plain_bucket)
         bucket['s3ql_passphrase'] = data_pw
-    else:    
+    else:
         data_pw = None
-        
+
     bucket = BetterBucket(data_pw, 'bzip2', plain_bucket)
 
     # Setup database
@@ -150,27 +150,27 @@ def main(args=None):
     param['max_obj_size'] = options.max_obj_size * 1024
     param['needs_fsck'] = False
     param['inode_gen'] = 0
-    param['max_inode'] = db.get_val('SELECT MAX(id) FROM inodes')   
-    param['last_fsck'] = time.time() - time.timezone
-    param['last-modified'] = time.time() - time.timezone
-    
+    param['max_inode'] = db.get_val('SELECT MAX(id) FROM inodes')
+    param['last_fsck'] = time.time()
+    param['last-modified'] = time.time()
+
     # This indicates that the convert_legacy_metadata() stuff
     # in BetterBucket is not required for this file system.
     param['bucket_revision'] = 1
-    
+
     log.info('Dumping metadata...')
     fh = tempfile.TemporaryFile()
-    dump_metadata(db, fh)            
+    dump_metadata(db, fh)
     def do_write(obj_fh):
         fh.seek(0)
         stream_write_bz2(fh, obj_fh)
         return obj_fh
-    
+
     log.info("Compressing and uploading metadata...")
     bucket.store('s3ql_seq_no_%d' % param['seq_no'], 'Empty')
     obj_fh = bucket.perform_write(do_write, "s3ql_metadata", metadata=param,
-                                  is_compressed=True) 
-    log.info('Wrote %.2f MB of compressed metadata.', obj_fh.get_obj_size() / 1024**2)
+                                  is_compressed=True)
+    log.info('Wrote %.2f MB of compressed metadata.', obj_fh.get_obj_size() / 1024 ** 2)
     pickle.dump(param, open(cachepath + '.params', 'wb'), 2)
 
 
